#include "dataarea.hh"

namespace
{
    static const unsigned char Empty = 0;
}

void DataArea::PrepareVec(vec& v, unsigned offset)
{
    if(offset >= v.size())
    {
        unsigned newsize = offset+1;
        if(v.capacity() < newsize)
        {
            const unsigned step = 2048;
            unsigned newcapacity = (newsize + step) &~ (step-1);
            v.reserve(newcapacity);
        }
        v.resize(newsize);
    }
}

DataArea::vec& DataArea::GetRef(unsigned offset, unsigned& base)
{
    map::iterator i = blobs.upper_bound(offset);
    if(i != blobs.end())
    {
        unsigned prev_top = i->first + i->second.size();
        if(prev_top <= offset)
        {
            base = i->first;
            return i->second;
        }
    }
    
    /* Create a new blob. */
    base = offset;
    return blobs[offset];
}

void DataArea::WriteByte(unsigned pos, unsigned char byte)
{
    unsigned base;
    vec& vector = GetRef(pos, base);
    unsigned vecpos = pos - base;
    PrepareVec(vector, vecpos);
    vector[vecpos] = byte;
}

unsigned char DataArea::GetByte(unsigned pos) const
{
    map::const_iterator i = blobs.upper_bound(pos);
    if(i != blobs.end())
    {
        unsigned prev_top = i->first + i->second.size();
        if(prev_top <= pos)
        {
            unsigned base = i->first;
            return i->second[pos - base];
        }
    }
    /* Nothing found. */
    return Empty;
}

unsigned DataArea::GetBase() const
{
    map::const_iterator i = blobs.begin();
    if(i == blobs.end()) return 0;
    return i->first;
}

unsigned DataArea::GetTop() const
{
    map::const_iterator i = blobs.end();
    if(i == blobs.begin()) return 0;
    --i;
    return i->first + i->second.size();
}

const std::vector<unsigned char> DataArea::GetContent() const
{
    std::vector<unsigned char> result(GetSize(), Empty);
    const unsigned base = GetBase();
    for(map::const_iterator i = blobs.begin(); i != blobs.end(); ++i)
    {
        unsigned pos = i->first - base;
        std::copy(i->second.begin(), i->second.end(), result.begin()+pos);
    }
    return result;
}
